
import argparse
import os

from axdscfg.utils import generate_key, encrypt_credentials_file

def main():

    parser = argparse.ArgumentParser(prog='axdscfg',
            description='Credentials from encrypted file package.')

    group = parser.add_mutually_exclusive_group()

    group.add_argument('-e', '--encrypt_key', action='store_true',
            help='Encrypt a file named credentials to credentials.enc')

    group.add_argument('-g', '--gen_key', action='store_true',
            help='Generates a key named axds_cfg.key')

    parser.add_argument('-d', '--dir', default='./',
            help='Specify a custom folder to save')

    args = parser.parse_args()

    config_path = args.dir

    if args.dir and not args.encrypt_key and not args.gen_key:
        print('Use -h for help')

    if args.encrypt_key:
        print('encrypting credentials...')
        config_path = os.path.abspath(config_path)
        encrypt_credentials_file(config_path)

    if args.gen_key:
        print('generating key...')
        config_path = os.path.abspath(config_path)
        generate_key(config_path)




from os.path import isfile

from axdscfg.exceptions import ConfigFileNotFoundError, \
    KeyFileNotFoundError

import nacl.secret
import nacl.utils

# Arquivo de chave de criptografia
KEY_FILE = '/axds_cfg.key'
# Arquivo de senhas criptografadas
CREDENTIALS_FILE = '/credentials'
PASSWORD_FILE = '/credentials.enc'

def generate_key(key_path):
    key_file = key_path + KEY_FILE

    key = nacl.utils.random(nacl.secret.SecretBox.KEY_SIZE)
    with open(key_file, 'wb') as outfile:
        outfile.write(key)

def encrypt_credentials_file(config_path):
    if not isfile(config_path + KEY_FILE):
        raise KeyFileNotFoundError

    with open(config_path + KEY_FILE, 'rb') as infile:
        key = infile.read()

    box = nacl.secret.SecretBox(key)

    credentials_infile = config_path + CREDENTIALS_FILE
    if not isfile(credentials_infile):
        raise ConfigFileNotFoundError

    credentials_outfile = config_path + PASSWORD_FILE
    with open(credentials_infile, 'rb') as infile:
        with open(credentials_outfile, 'wb') as outfile:
            outfile.write(box.encrypt(infile.read()))




# Exceptions
class Error(Exception):
    """ Base class for exceptions """
    pass

class ConfigFileNotFoundError(Error):
    """ O arquivo de configuração não foi encontrado! """
    pass

class KeyFileNotFoundError(Error):
    """ A chave secreta não foi encontrada! """
    pass




from axdscfg.exceptions import ConfigFileNotFoundError, \
    KeyFileNotFoundError

import nacl.secret
from os.path import isfile
from configparser import RawConfigParser

# Arquivo de chave de criptografia
KEY_FILE = '/axds_cfg.key'
# Arquivo de senhas criptografadas
PASSWORD_FILE = '/credentials.enc'

class Credentials():
    CUSTOM_CONFIG_PATH = None
    CRYPT_KEY = None
    params = None

    # Se os arquivos de configuração estiverem em outro diretório,
    #    o diretório deverá ser passado na chamada da classe
    def __init__(self, config_path):
        self.CUSTOM_CONFIG_PATH = config_path
        self.CRYPT_KEY = self.CUSTOM_CONFIG_PATH + KEY_FILE

        if not isfile(self.CRYPT_KEY):
            raise KeyFileNotFoundError

        PARAMS_FILE = self.CUSTOM_CONFIG_PATH + PASSWORD_FILE

        if not isfile(PARAMS_FILE):
            raise ConfigFileNotFoundError

        self.params = self.__read_config(PARAMS_FILE)

    def __read_config(self, p_file):
        config_parser = RawConfigParser()

        # Chamada da função de decriptação
        config_file_binary = self.__decrypt_file(self.CRYPT_KEY, p_file)

        config_parser.read_string(config_file_binary.decode('utf-8'))
        return config_parser

    def __decrypt_file(self, key_file, in_filename):
        """ Decrypts a file using PyNaCl Secret Box.
            The method uses a key file to decrypt the credentials file.
        """
        with open(key_file, 'rb') as infile:
            key = infile.read()

        box = nacl.secret.SecretBox(key)

        with open(in_filename, 'rb') as infile:
            credentials_file_binary = box.decrypt(infile.read())

        return credentials_file_binary

    def get_config_param(self, param_section, param_key):
        return self.params.get(param_section, param_key)




import setuptools
import axdscfg

requirements = ["configparser>=3.7.4",
                "PyNaCl==1.3.0"] 

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=axdscfg.__title__,
    version=axdscfg.__version__,
    author=axdscfg.__author__,
    author_email=axdscfg.__email__,
    description=axdscfg.__summary__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'axdscfg = axdscfg:main'
        ]
    },
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: Unix",
    ],
    python_requires='>=3.5',
)



from axdscfg.axdscfg import main

__all__ = [
    "__title__", "__summary__", "__version__", "__author__",
    "__email__", "__license__",
]

__title__ = "axdscfg"
__summary__ = ("Python lib to get credentials from encrypted file.")

__version__ = "1.0.0"

__author__ = "Axiomas Data Science"
__email__ = "suporte@axiomas.tech"

__license__ = "GNU GENERAL PUBLIC LICENSE 3"
